////////////////////////////////////////////////////////////////////////
// SLump - A doom wad/lump manager                                    //
// By Simon Judd, 2006                                                //
// ------------------------------------------------------------------ //
// main.cpp - Main app stuff                                          //
////////////////////////////////////////////////////////////////////////

// INCLUDES ////////////////////////////////////////////////////////////
#include "main.h"
#include "main_window.h"
#include "text_area.h"

#include <wx/image.h>
#include <wx/sysopt.h>
#include <wx/stdpaths.h>

// VARIABLES ///////////////////////////////////////////////////////////
MainWindow	*main_window;
Wad			iwad;
Wad			clipboard;


// EXTERNAL VARIABLES //////////////////////////////////////////////////
EXTERN_CVAR(String, iwad_path)


IMPLEMENT_APP(MainApp)

// str: Converts a wxString to a std::string, converting from unicode to ASCII if necessary
// ----------------------------------------------------------------------------------------
std::string str(wxString in)
{
	wxCharBuffer buffer = in.ToAscii();
	std::string ret = (const char*)buffer;
	return ret;
}

// save_config: Saves the main configuration file
// ----------------------------------------------
void save_config()
{
	FILE *fp = fopen("slumped.cfg", "wt");
	save_cvars(fp);
	fclose(fp);
}

// load_config: Loads the main configuration file
// ----------------------------------------------
void load_config()
{
	Tokenizer tz;
	if (!tz.open_file(_T("slumped.cfg"), 0, 0))
		return;

	string token = tz.get_token();

	while (token != _T("!END"))
	{
		if (token == _T("cvars"))
			load_cvars(&tz);

		token = tz.get_token();
	}
}

// MainApp::OnInit: Called when the application begins
// ---------------------------------------------------
bool MainApp::OnInit()
{
	wxLog::SetActiveTarget(new wxLogStderr(fopen("slumped.log", "wt")));
	wxSystemOptions::SetOption(_T("msw.remap"), 0);
	wxImage::AddHandler(new wxPNGHandler);

	wxStandardPaths sp;
	wxString dataDir = sp.GetDataDir();
	wxSetWorkingDirectory(dataDir);

	load_config();
	load_languages();

	main_window = new MainWindow();
	SetTopWindow(main_window);

	if (iwad_path == "")
	{
		if (wxMessageBox(_T("To view graphics properly, you must set an IWAD to get the palette from\n(it can be changed later from the file menu)"), _T("SLumpEd"), wxOK|wxCANCEL) == wxOK)
		{
			string filename = wxFileSelector(_T("Choose file(s) to open"), _T(""), _T(""), _T(""),
											_T("Doom Wad files (*.wad)|*.wad"), wxOPEN|wxFILE_MUST_EXIST);

			if (filename != _T(""))
				iwad_path = filename;
		}
	}

	iwad.open(iwad_path);

	if (argc > 0)
	{
		for (int a = 0; a < argc; a++)
			wxLogMessage(argv[a]);

		for (int a = 1; a < argc; a++)
		{
			wxString arg = argv[a];

			if (arg.Right(4).CmpNoCase(_T(".wad")) ||
				arg.Right(4).CmpNoCase(_T(".zip")) ||
				arg.Right(4).CmpNoCase(_T(".pk3")))
			{
				wxLogMessage(_T("Opening wad %s"), arg);
				//wads.open_wad(wx_to_str(arg));
				//wads_opened = true;
				main_window->openWad(arg);
			}
		}
	}

	return true;
}

// MainApp::OnExit: Called when the application exits
// --------------------------------------------------
int MainApp::OnExit()
{
	save_config();

	return 0;
}
