
#include "main.h"
#include "misc.h"

void image_from_lump(Lump* lump, wxImage* image, wxColour* palette)
{
	if (lump->getType() == LUMP_FLAT || lump->getType() == LUMP_GFX2)
	{
		if (lump->getSize() == 64*64)
			image->Create(64, 64, true);
		else if (lump->getSize() == 320*200)
			image->Create(320, 200, true);
		else if (lump->getSize() == 64*128)
			image->Create(64, 128, true);
		else
			return;

		for (int a = 0; a < lump->getSize(); a++)
		{
			BYTE* data = image->GetData();
			BYTE col = lump->getData()[a];
			data[a*3] = palette[col].Red();
			data[(a*3)+1] = palette[col].Green();
			data[(a*3)+2] = palette[col].Blue();
		}
	}

	if (lump->getType() == LUMP_PATCH || lump->getType() == LUMP_SPRITE || lump->getType() == LUMP_GFX)
	{
		// Get header & offsets
		patch_header_t *header = (patch_header_t *)lump->getData();
		long *col_offsets= (long *)((BYTE *)lump->getData() + sizeof(patch_header_t));

		image->Create(header->width, header->height, true);
		image->SetAlpha(NULL);
		memset(image->GetAlpha(), 0, header->width * header->height);

		// Read data
		for (int c = 0; c < header->width; c++)
		{
			// Go to start of column
			BYTE* data = lump->getData();
			data += col_offsets[c];

			// Read posts
			while (1)
			{
				// Get row offset
				BYTE row = *data;

				if (row == 255) // End of column?
					break;
				
				// Get no. of pixels
				data++;
				BYTE n_pix = *data;

				data++; // Skip buffer
				for (BYTE p = 0; p < n_pix; p++)
				{
					data++;
					image->SetRGB(c, row+p, palette[*data].Red(), palette[*data].Green(), palette[*data].Blue());
					image->SetAlpha(c, row+p, 255);
				}
				data += 2; // Skip buffer & go to next row offset
			}
		}
	}

	if (lump->getType() == LUMP_PNG)
	{
		lump->dumpToFile(_T("slumptemp"));
		image->LoadFile(_T("slumptemp"), wxBITMAP_TYPE_PNG);
		remove("slumptemp");
	}
}

// CRC stuff for PNG

/* Table of CRCs of all 8-bit messages. */
unsigned long crc_table[256];

/* Flag: has the table been computed? Initially false. */
int crc_table_computed = 0;

/* Make the table for a fast CRC. */
void make_crc_table(void)
{
	unsigned long c;
	int n, k;

	for (n = 0; n < 256; n++)
	{
		c = (unsigned long) n;

		for (k = 0; k < 8; k++)
		{
			if (c & 1)
				c = 0xedb88320L ^ (c >> 1);
			else
				c = c >> 1;
		}

		crc_table[n] = c;
	}

	crc_table_computed = 1;
}

/* Update a running CRC with the bytes buf[0..len-1]--the CRC
should be initialized to all 1's, and the transmitted value
is the 1's complement of the final running CRC (see the
crc() routine below)). */
unsigned long update_crc(unsigned long crc, unsigned char *buf, int len)
{
	unsigned long c = crc;
	int n;

	if (!crc_table_computed)
		make_crc_table();

	for (n = 0; n < len; n++)
		c = crc_table[(c ^ buf[n]) & 0xff] ^ (c >> 8);

	return c;
}

/* Return the CRC of the bytes buf[0..len-1]. */
unsigned long crc(unsigned char *buf, int len)
{
	return update_crc(0xffffffffL, buf, len) ^ 0xffffffffL;
}
