////////////////////////////////////////////////////////////////////////
// SLump - A doom wad/lump manager                                    //
// By Simon Judd, 2006                                                //
// ------------------------------------------------------------------ //
// wad.cpp - Wadfile/lump handing functions                           //
////////////////////////////////////////////////////////////////////////

// INCLUDES ////////////////////////////////////////////////////////////
#include "main.h"

#include <wx/wfstream.h>
#include <wx/zipstrm.h>
#include <wx/ptr_scpd.h>

// VARIABLES ///////////////////////////////////////////////////////////
string map_lumps[12] =
{
	_T("THINGS"),
	_T("VERTEXES"),
	_T("LINEDEFS"),
	_T("SIDEDEFS"),
	_T("SECTORS"),
	_T("SEGS"),
	_T("SSECTORS"),
	_T("NODES"),
	_T("BLOCKMAP"),
	_T("REJECT"),
	_T("SCRIPTS"),
	_T("BEHAVIOR")
};

CVAR(Bool, determine_type_load, false, CVAR_SAVE)

// EXTERNAL VARIABLES //////////////////////////////////////////////////

/*
BYTE* Lump::Data()
{
	if (!loaded && size != 0 && parent)
	{
		if (!data)
			data = (BYTE*)malloc(size);

		FILE* fp = fopen(parent->path.c_str(), "rb");
		fseek(fp, offset, SEEK_SET);
		fread(data, size, 1, fp);
		fclose(fp);

		loaded = true;
	}

	return data;
}

// Lump::LoadFile: Loads a file to a lump
// --------------------------------------
bool Lump::LoadFile(string filename)
{
	FILE *fp = fopen((char*)filename.c_str(), "rb");

	if (!fp)
		return false;

	// Get size
	fseek(fp, 0, SEEK_END);
	long filesize = ftell(fp);
	this->size = filesize;
	fseek(fp, 0, SEEK_SET);

	// Load data
	BYTE* buffer = (BYTE*)malloc(filesize);
	fread(buffer, filesize, 1, fp);
	Resize(filesize);
	memcpy(this->data, buffer, filesize);

	// Finish up
	free(buffer);
	fclose(fp);

	changed = true;

	return true;
}

string Lump::GetType()
{
	if (type == LUMP_MARKER) return _T("Marker");
	if (type == LUMP_TEXT) return _T("Text");

	if (type == LUMP_PATCH) return _T("Patch");
	if (type == LUMP_SPRITE) return _T("Sprite");
	if (type == LUMP_FLAT) return _T("Flat");
	if (type == LUMP_GFX) return _T("Graphic");
	if (type == LUMP_GFX2) return _T("Graphic");
	if (type == LUMP_PNG) return _T("PNG");

	if (type == LUMP_SOUND) return _T("Sound");
	if (type == LUMP_MUS) return _T("Music (MUS)");
	if (type == LUMP_MIDI) return _T("Music (MIDI)");
	if (type == LUMP_MOD) return _T("Music (Module)");

	if (type == LUMP_TEXTURES) return _T("TEXTUREx");
	if (type == LUMP_PNAMES) return _T("PNames");

	if (type == LUMP_MAP) return _T("Map");
	if (type == LUMP_LINEDEFS) return _T("Map Linedefs");
	if (type == LUMP_SIDEDEFS) return _T("Map Sidedefs");
	if (type == LUMP_VERTEXES) return _T("Map Vertices");
	if (type == LUMP_SECTORS) return _T("Map Sectors");
	if (type == LUMP_THINGS) return _T("Map Things");

	return _T("Unknown");
}
*/

Lump::Lump(long offset, long size, string name, Wad* parent)
{
	this->offset = offset;
	this->size = size;
	this->name = name;
	this->parent = parent;

	changed = false;
	type = LUMP_UNKNOWN;
	data = NULL;
	loaded = false;

	if (size > 0)
	{
		data = new BYTE[size];
		memset(data, 0, size);
	}
}

Lump::~Lump()
{
	if (data)
		delete[] data;
}

BYTE* Lump::getData()
{
	if (loaded)
		return data;

	if (size > 0 && parent)
	{
		FILE* fp = fopen(parent->path.c_str(), "rb");
		fseek(fp, offset, SEEK_SET);
		fread(data, size, 1, fp);
		fclose(fp);

		loaded = true;
	}

	return data;
}

string Lump::getName(bool full)
{
	if (full && directory.size() > 0)
		return getFullDir() + name;
	else
		return name;
}

void Lump::setOffset(DWORD o)
{
	offset = o;
	changed = true;
}

void Lump::reSize(DWORD s)
{
	if (data)
	{
		delete[] data;
		data = NULL;
		loaded = false;
	}

	size = s;

	if (size > 0)
	{
		data = new BYTE[size];
		memset(data, 0, size);
	}

	changed = true;
}

void Lump::setName(string n)
{
	name = n;
	changed = true;
}

void Lump::dumpToFile(string filename)
{
	if (!data)
		return;

	FILE *fp = fopen((char*)filename.c_str(), "wb");
	fwrite(data, size, 1, fp);
	fclose(fp);
}

bool Lump::loadFile(string filename)
{
	FILE *fp = fopen((char*)filename.c_str(), "rb");

	if (!fp)
		return false;

	// Get size
	fseek(fp, 0, SEEK_END);
	long filesize = ftell(fp);
	size = filesize;
	fseek(fp, 0, SEEK_SET);

	reSize(size);
	fread(data, size, 1, fp);
	fclose(fp);

	loaded = true;
	changed = true;

	return true;
}

void Lump::loadStream(FILE *fp)
{
	loaded = true;

	if (size == 0)
		return;

	fseek(fp, offset, SEEK_SET);
	fread(data, size, 1, fp);
}

void Lump::loadData(BYTE* buf, DWORD size)
{
	reSize(size);
	memcpy(data, buf, size);
	loaded = true;
	changed = true;
}

string Lump::getEx(string key)
{
	for (int a = 0; a < ex_info.size(); a++)
	{
		if (ex_info[a].key == key)
			return ex_info[a].value;
	}

	return _T("");
}

void Lump::setEx(string key, string val)
{
	for (int a = 0; a < ex_info.size(); a++)
	{
		if (ex_info[a].key == key)
		{
			ex_info[a].value = val;
			return;
		}
	}

	key_value_t kv;
	kv.key = key;
	kv.value = val;

	ex_info.push_back(kv);
}

void Lump::determineType(bool force)
{
	if (!force && type != LUMP_UNKNOWN)
		return;

	if (type == LUMP_FOLDER)
		return;

	if (parent)
	{
		if (parent->zip)
		{
			wxFileName fn(getName(), wxPATH_UNIX);
			string ext = fn.GetExt();

			if (ext == _T("txt"))
				type = LUMP_TEXT;

			if (ext == _T("png"))
				type = LUMP_PNG;

			if (ext == _T("wad"))
				type = LUMP_WAD;

			if (fn.GetName().CmpNoCase(_T("animdefs")) == 0)
				setEx(_T("TextFormat"), _T("ANIMDEFS"));

			if (fn.GetName().CmpNoCase(_T("sndinfo")) == 0)
				setEx(_T("TextFormat"), _T("SNDINFO"));

			if (fn.GetName().CmpNoCase(_T("language")) == 0)
				type = LUMP_TEXT;

			if (directory.size() > 0)
			{
				if (directory[0].CmpNoCase(_T("patches")) == 0 && type == LUMP_UNKNOWN)
				{
					type = LUMP_PATCH;
					return;
				}

				if (directory[0].CmpNoCase(_T("sprites")) == 0 && type == LUMP_UNKNOWN)
				{
					type = LUMP_SPRITE;
					return;
				}

				if (directory[0].CmpNoCase(_T("flats")) == 0 && type == LUMP_UNKNOWN)
				{
					type = LUMP_FLAT;
					return;
				}

				if (directory[0].CmpNoCase(_T("sounds")) == 0 && type == LUMP_UNKNOWN)
				{
					type = LUMP_SOUND;
					return;
				}

				if (directory[0].CmpNoCase(_T("actors")) == 0)
					setEx(_T("TextFormat"), _T("DECORATE"));

				if (directory[0].CmpNoCase(_T("mapinfo")) == 0)
					setEx(_T("TextFormat"), _T("MAPINFO"));
			}
		}
	}

	// Marker
	if (size == 0 && type != LUMP_MAP)
	{
		type = LUMP_MARKER;
		return;
	}

	// TEXTUREx
	if (name == _T("TEXTURE1") || name == _T("TEXTURE2"))
	{
		type = LUMP_TEXTURES;
		return;
	}

	// PNames
	if (name == _T("PNAMES"))
	{
		type = LUMP_PNAMES;
		return;
	}

	// THINGS
	if (name == _T("THINGS"))
	{
		if (size % 10 == 0)
		{
			type = LUMP_THINGS;
			return;
		}
	}

	// LINEDEFS
	if (name == _T("LINEDEFS"))
	{
		if (size % 14 == 0 || size % 16 == 0)
		{
			type = LUMP_LINEDEFS;
			return;
		}
	}

	// SIDEDEFS
	if (name == _T("SIDEDEFS"))
	{
		if (size % 30 == 0)
		{
			type = LUMP_SIDEDEFS;
			return;
		}
	}

	// SECTORS
	if (name == _T("SECTORS"))
	{
		if (size % 26 == 0)
		{
			type = LUMP_SECTORS;
			return;
		}
	}

	// VERTEXES
	if (name == _T("VERTEXES"))
	{
		if (size % 4 == 0)
		{
			type = LUMP_VERTEXES;
			return;
		}
	}

	// PNG
	if (size > 8)
	{
		if (data[0] == 137 && data[1] == 80 &&
			data[2] == 78 && data[3] == 71 &&
			data[4] == 13 && data[5] == 10 &&
			data[6] == 26 && data[7] == 10)
		{
			type = LUMP_PNG;
			return;
		}
	}

	// WAD
	if (size >= 12)
	{
		if (data[1] == 'W' && data[2] == 'A' && data[3] == 'D' &&
			(data[0] == 'P' || data[0] == 'I'))
		{
			type = LUMP_WAD;
			return;
			/*
			short dir_offset = 0;
			short numlumps = 0;
			memcpy(&numlumps, data + 4, 4);
			memcpy(&dir_offset, data + 8, 4);

			if ((short)(data + dir_offset) >= 0 && (short)(data + dir_offset) <= size)
			{
				type = LUMP_WAD;
				return;
			}
			*/
		}
	}

	// MUS
	if (size > 16)
	{
		if (data[0] == 'M' && data[1] == 'U' && data[2] == 'S')
		{
			type = LUMP_MUS;
			return;
		}
	}

	// Patch
	if (parent->lumpIndex(this) > parent->patches[START] && parent->lumpIndex(this) < parent->patches[END])
	{
		type = LUMP_PATCH;
		return;
	}

	// Flat
	if (parent->lumpIndex(this) > parent->flats[START] && parent->lumpIndex(this) < parent->flats[END])
	{
		type = LUMP_FLAT;
		return;
	}

	// Sprite
	if (parent->lumpIndex(this) > parent->sprites[START] && parent->lumpIndex(this) < parent->sprites[END])
	{
		type = LUMP_SPRITE;
		return;
	}

	// Misc Gfx
	if ((name == _T("TITLE") || name == _T("HELP1") ||
		name == _T("HELP2") || name == _T("CREDIT") ||
		name == _T("FINALE1") || name == _T("FINALE2") ||
		name == _T("FINALE3") || name == _T("INTERPIC")) &&
		size == 320*200)
	{
		type = LUMP_GFX2;
		return;
	}

	// Detect doom patch format
	if (size > sizeof(patch_header_t))
	{
		patch_header_t *header = (patch_header_t *)data;

		if (header->height > 0 && header->height < 4096 &&
			header->width > 0 && header->width < 4096 &&
			header->top > -2000 && header->top < 2000 &&
			header->left > -2000 && header->left < 2000)
		{
			long *col_offsets = (long *)((BYTE *)data + sizeof(patch_header_t));
			
			if (size < sizeof(patch_header_t) + (header->width * sizeof(long)))
			{
				//wxLogMessage("lump %s not a patch, col_offsets error 1", lump->Name().c_str());
				return;
			}

			for (int a = 0; a < header->width; a++)
			{
				if (col_offsets[a] > size || col_offsets[a] < 0)
				{
					//wxLogMessage("lump %s not a patch, col_offsets error 2", lump->Name().c_str());
					return;
				}
			}

			type = LUMP_GFX;
		}
	}
}

string Lump::getTypeString()
{
	if (type == LUMP_MARKER) return _T("Marker");
	if (type == LUMP_TEXT) return _T("Text");

	if (type == LUMP_PATCH) return _T("Patch");
	if (type == LUMP_SPRITE) return _T("Sprite");
	if (type == LUMP_FLAT) return _T("Flat");
	if (type == LUMP_GFX) return _T("Graphic");
	if (type == LUMP_GFX2) return _T("Graphic");
	if (type == LUMP_PNG) return _T("PNG");

	if (type == LUMP_SOUND) return _T("Sound");
	if (type == LUMP_MUS) return _T("Music (MUS)");
	if (type == LUMP_MIDI) return _T("Music (MIDI)");
	if (type == LUMP_MOD) return _T("Music (Module)");

	if (type == LUMP_TEXTURES) return _T("TEXTUREx");
	if (type == LUMP_PNAMES) return _T("PNames");

	if (type == LUMP_MAP) return _T("Map");
	if (type == LUMP_LINEDEFS) return _T("Map Linedefs");
	if (type == LUMP_SIDEDEFS) return _T("Map Sidedefs");
	if (type == LUMP_VERTEXES) return _T("Map Vertices");
	if (type == LUMP_SECTORS) return _T("Map Sectors");
	if (type == LUMP_THINGS) return _T("Map Things");

	if (type == LUMP_WAD) return _T("Wad File");

	if (type == LUMP_FOLDER) return _T("Dir");

	return _T("Unknown");
}

void Lump::addDir(string name, int index)
{
	if (index == -1)
		directory.push_back(name);
	else
		directory.insert(directory.begin() + index, name);
}

string Lump::getDir(int index)
{
	if (index < 0 || index > directory.size())
		return _T("");
	else
		return directory[index];
}

void Lump::renameDir(string nname, int index)
{
	if (index >= 0 && index < directory.size())
		directory[index] = nname;
}

string Lump::getFullDir()
{
	if (directory.size() == 0)
		return _T("");

	string ret;
	for (int a = 0; a < directory.size(); a++)
	{
		ret += directory[a];
		ret += _T("/");
	}

	return ret;
}

void Wad::findMaps()
{
	// Find all maps
	for (DWORD l = 0; l < numLumps(); l++)
	{
		bool done = false;
		bool existing_map_lumps[12] = { false, false, false, false,
			false, false, false, false,
			false, false, false, false };

		int index = l;
		while (!done)
		{
			l++;

			if (l == numLumps())
				break;

			string name = directory[l]->getName();
			done = true;

			for (int s = 0; s < 12; s++)
			{
				if (name == map_lumps[s])
				{
					existing_map_lumps[s] = true;
					done = false;
				}
			}
		}

		l--;

		if (!memchr(existing_map_lumps, 0, 5))
			directory[index]->setType(LUMP_MAP);
	}
}

// Wad::open: Opens a wad file
// ---------------------------
bool Wad::open(string filename, bool load_data)
{
	if (filename == _T(""))
		return false;

	close();
	path = filename;

	// Open the file
	FILE *fp = fopen(str(filename).c_str(), "rb");

	// Check if it opened correctly
	if (!fp)
	{
		//wxMessageBox(wxString::Format(_T("Wad file \"%s\" cannot be found.\n"), filename.c_str()));
		wxLogMessage(_T("Wad file \"%s\" cannot be found.\n"), filename.c_str());
		close();
		return false;
	}

	long num_lumps = 0;
	long filesize = 0;
	fseek(fp, 0, SEEK_END);
	filesize = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	// Read the header
	fread(&type, 1, 4, fp);			// Wad type
	fread(&num_lumps, 4, 1, fp);	// No. of lumps in wad
	fread(&dir_offset, 4, 1, fp);	// Offset to directory

	// Lock the wad if IWAD
	if (type[0] == 'I')
		locked = true;
	else
		locked = false;

	if (type[1] != 'W' || type[2] != 'A' || type[3] != 'D')
	{
		wxMessageBox(_T("Not a valid Doom wad file!"), _T("Error"), wxICON_ERROR);
		return false;
	}

	// Setup the directory
	//directory = (dir_entry_t *)calloc(num_lumps, sizeof(dir_entry_t));
	//directory = (Lump **)calloc(num_lumps, sizeof(Lump*));

	// Read the directory
	fseek(fp, dir_offset, SEEK_SET);
	for (DWORD d = 0; d < num_lumps; d++)
	{
		char name[9] = "";
		DWORD offset = 0;
		DWORD size = 0;

		fread(&offset, 4, 1, fp);	// Offset
		fread(&size, 4, 1, fp);		// Size
		fread(name, 1, 8, fp);		// Name

		if (offset + size > filesize)
		{
			wxMessageBox(_T("Not a valid Doom wad file!"), _T("Error"), wxICON_ERROR);
			return false;
		}

		name[8] = '\0';
		Lump* nlump = new Lump(offset, size, wxString::FromAscii(name).Upper(), this);

		if (nlump->getName() == _T("P_START") || nlump->getName() == _T("PP_START"))
			patches[START] = d;

		if (nlump->getName() == _T("P_END") || nlump->getName() == _T("PP_END"))
			patches[END] = d;

		if (nlump->getName() == _T("F_START") || nlump->getName() == _T("FF_START"))
			flats[START] = d;

		if (nlump->getName() == _T("F_END") || nlump->getName() == _T("FF_END"))
			flats[END] = d;

		if (nlump->getName() == _T("S_START") || nlump->getName() == _T("SS_START"))
			sprites[START] = d;

		if (nlump->getName() == _T("S_END") || nlump->getName() == _T("SS_END"))
			sprites[END] = d;

		if (nlump->getName() == _T("TX_START"))
			tx[START] = d;

		if (nlump->getName() == _T("TX_END"))
			tx[END] = d;

		directory.push_back(nlump);
	}

	// Read wad data into memory
	if (load_data)
	{
		for (DWORD d = 0; d < numLumps(); d++)
			directory[d]->loadStream(fp);
	}

	if (determine_type_load)
	{
		for (DWORD d = 0; d < numLumps(); d++)
			directory[d]->determineType();
	}

	fclose(fp);

	findMaps();

	zip = false;
	changed = false;
	return true;
}

void Wad::addDir(string path)
{
	for (int a = 0; a < dirs.size(); a++)
	{
		if (dirs[a] == path)
			return;
	}

	dirs.push_back(path);
}

bool Wad::openZip(string filename, bool load_data)
{
	path = filename;
	this->zip = true;

	wxZipEntry* entry;

	wxFFileInputStream in(filename);
	if (!in.IsOk())
		return false;

	wxZipInputStream zip(in);
	if (!zip.IsOk())
	{
		wxMessageBox(_T("Invalid zip file!"));
		return false;
	}

	entry = zip.GetNextEntry();
	while (entry)
	{
		if (!entry->IsDir())
		{
			wxFileName fn(entry->GetName(wxPATH_UNIX), wxPATH_UNIX);

			// Create lump
			Lump *nlump = new Lump(0, entry->GetSize(), fn.GetFullName()/*entry->GetName(wxPATH_UNIX)*/, this);

			// Load data
			BYTE* buffer = new BYTE[entry->GetSize()];
			zip.Read(buffer, entry->GetSize());
			nlump->loadData(buffer, entry->GetSize());
			delete[] buffer;

			// Get directory info
			wxArrayString dirs = fn.GetDirs();
			for (int a = 0; a < dirs.size(); a++)
				nlump->addDir(dirs[a]);

			if (dirs.size() > 0)
			{
				string dir = dirs[0] + _T("/");
				addDir(dir);
				for (int a = 1; a < dirs.size(); a++)
				{
					dir += dirs[a] + _T("/");
					addDir(dir);
				}
			}

			// Add to directory
			nlump->setChanged(false);
			directory.push_back(nlump);

			// Determine type
			if (determine_type_load)
				nlump->determineType();
		}
		else
			addDir(entry->GetName(wxPATH_UNIX));

		entry = zip.GetNextEntry();
	}

	for (int a = 0; a < this->dirs.size(); a++)
	{
		wxFileName fn(this->dirs[a], wxPATH_UNIX);
		wxArrayString dirs2 = fn.GetDirs();

		//Lump* lump = addLump(this->dirs[a], 0);
		Lump* lump = addLump(dirs2[fn.GetDirCount()-1] + _T("/"), 0);

		for (int a = 0; a < dirs2.size()-1; a++)
			lump->addDir(dirs2[a]);

		lump->setType(LUMP_FOLDER);
	}

	findMaps();

	changed = false;
	return true;
}

// Wad::dump_directory: Writes all directory entries to the logfile
// ----------------------------------------------------------------
void Wad::dumpDirectory()
{
	for (DWORD l = 0; l < numLumps(); l++)
		wxLogMessage(_T("%d: %s (%db at %d)\n"), l, directory[l]->getName(), directory[l]->getSize(), directory[l]->getOffset());
}

// Wad::get_lump_index: Returns the index of the first lump with the specified name
//                      Returns -1 if no matching lump is found
// --------------------------------------------------------------------------------
long Wad::getLumpIndex(string name, DWORD offset)
{
	if (numLumps() == 0 || offset > numLumps())
		return -1;

	for (DWORD l = offset; l < numLumps(); l++)
	{
		if (directory[l]->getName().CmpNoCase(name) == 0)
			return l;
	}

	return -1;
}

// Wad::get_lump: Returns a pointer to the first matching lump from offset
// -----------------------------------------------------------------------
Lump* Wad::getLump(string name, DWORD offset)
{
	for (DWORD l = offset; l < numLumps(); l++)
	{
		if (directory[l]->getName().CmpNoCase(name) == 0)
			return directory[l];
	}

	return NULL;
}

// Wad::add_lump: Adds a new 0-sized lump before <index>
// -----------------------------------------------------
Lump* Wad::addLump(string name, long index)
{
	Lump* nlump = new Lump(0, 0, name, this);

	if (index < 0 || index >= numLumps())
		directory.push_back(nlump);
	else
		directory.insert(directory.begin() + index, nlump);

	changed = true;
	return nlump;
}

// Wad::replace_lump: Replaces lump data with new data
// ---------------------------------------------------
void Wad::replaceLump(string name, DWORD new_size, BYTE *data, DWORD offset)
{
	int index = getLumpIndex(name, offset);

	if (index > -1)
	{
		directory[index]->reSize(new_size);
		memcpy(directory[index]->getData(), data, new_size);
	}

	changed = true;
}

// Wad::delete_lump: Deletes a lump
// --------------------------------
void Wad::deleteLump(string name, DWORD offset)
{
	long index = getLumpIndex(name, offset);
	deleteLump(index);
}

// Wad::delete_lump: Deletes a lump
// --------------------------------
void Wad::deleteLump(long index, bool delfolder)
{
	if (index < 0 || index > numLumps())
		return;

	Lump *lump = lumpAt(index);
	if (lump->getType() == LUMP_FOLDER && delfolder)
	{
		vector<string> dirs;
		for (int a = 0; a < lump->dirLevel(); a++)
			dirs.push_back(lump->getDir(a));

		string ldir = lump->getName(false);
		ldir.RemoveLast();
		dirs.push_back(ldir);

		for (int a = 0; a < numLumps(); a++)
		{
			if (lumpAt(a)->dirLevel() < dirs.size())
				continue;

			bool in = true;
			for (int d = 0; d < dirs.size(); d++)
			{
				if (dirs[d] != lumpAt(a)->getDir(d))
				{
					in = false;
					break;
				}
			}

			if (in)
			{
				deleteLump(a, false);
				a--;
			}
		}

		index = lumpIndex(lump);
	}

	directory.erase(directory.begin() + index);

	changed = true;
}

// copy_file: Creates a copy of a file
// -----------------------------------
void copy_file(string input, string output)
{
	FILE *in = fopen(str(input).c_str(), "rb");
	if (in)
	{
		FILE *out = fopen(str(output).c_str(), "wb");
		while (!feof(in))
		{
			BYTE b = 0;
			fread(&b, 1, 1, in);
			fwrite(&b, 1, 1, out);
		}
		fclose(out);
		fclose(in);
	}
}

// Wad::save: Saves a wad file to disk (backs up the file, then overwrites it)
// ---------------------------------------------------------------------------
void Wad::save()
{
	if (locked)
	{
		wxMessageBox(_T("Saving to an IWAD is disallowed. Use 'Save As' instead."), _T("Not Allowed"));
		return;
	}

	if (zip)
	{
		//wxMessageBox(_T("Saving zips not yet supported"), _T("Coming Soon ;)"));
		saveZip();
		return;
	}

	if (!parent)
	{
		string bakfile = path + _T(".bak");
		//wxLogMessage("%s\n%s\n", path.c_str(), bakfile.c_str());

		// Remove old backup file
		remove(str(bakfile).c_str());

		// Copy current file contents to new backup file
		copy_file(path, bakfile);
	}

	// Determine directory offset & individual lump offsets
	dir_offset = 12;

	for (DWORD l = 0; l < numLumps(); l++)
	{
		directory[l]->setOffset(dir_offset);
		dir_offset += directory[l]->getSize();
	}

	// Open wadfile for writing
	FILE *fp = NULL;

	if (!parent)
		fp = fopen(str(path).c_str(), "wb");
	else
		fp = fopen("slumptemp.wad", "wb");

	// Write the header
	long num_lumps = numLumps();
	fwrite(type, 1, 4, fp);
	fwrite(&num_lumps, 4, 1, fp);
	fwrite(&dir_offset, 4, 1, fp);

	// Write the lumps
	for (DWORD l = 0; l < num_lumps; l++)
		fwrite(directory[l]->getData(), directory[l]->getSize(), 1, fp);

	// Write the directory
	for (DWORD l = 0; l < numLumps(); l++)
	{
		char name[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
		long offset = directory[l]->getOffset();
		long size = directory[l]->getSize();

		for (int c = 0; c < directory[l]->getName().length(); c++)
			name[c] = directory[l]->getName()[c];

		fwrite(&offset, 4, 1, fp);
		fwrite(&size, 4, 1, fp);
		fwrite(name, 1, 8, fp);

		directory[l]->setChanged(false);
	}

	fclose(fp);

	if (parent)
	{
		parent->loadFile(_T("slumptemp.wad"));
		remove("slumptemp.wad");
	}

	changed = false;
}

void Wad::saveZip()
{
	string bakfile = path + _T(".bak");

	// Remove old backup file
	remove(str(bakfile).c_str());

	// Copy current file contents to new backup file
	copy_file(path, bakfile);

	wxFFileOutputStream out(path);
	if (!out.IsOk())
		return;

	wxZipOutputStream zip(out, 9);

	for (int a = 0; a < directory.size(); a++)
	{
		directory[a]->setChanged(false);

		if (directory[a]->getType() == LUMP_FOLDER)
			zip.PutNextDirEntry(directory[a]->getName());
		else
		{
			wxZipEntry* entry = new wxZipEntry(directory[a]->getName());

			zip.PutNextEntry(entry);
			zip.Write(directory[a]->getData(), directory[a]->getSize());
		}
	}

	zip.Close();
	changed = false;
}

// Wad::close: Closes the wad file
// -------------------------------
void Wad::close()
{
	if (path == _T(""))
		return;

	available_maps.clear();

	for (int a = 0; a < directory.size(); a++)
		delete directory[a];

	//free(directory);
	directory.clear();

	path = _T("");
	locked = true;
}

// Wad::needs_save: Returns true if the wad file is changed and needs saving
// -------------------------------------------------------------------------
bool Wad::needsSave()
{
	for (int a = 0; a < numLumps(); a++)
	{
		if (directory[a]->isChanged())
			changed = true;
	}

	return changed;
}

Lump* Wad::lumpAt(int index)
{
	if (index < 0 || index >= numLumps())
		 return NULL;

	return directory[index];
}

long Wad::lumpIndex(Lump* lump)
{
	for (int a = 0; a < directory.size(); a++)
	{
		if (directory[a] == lump)
			return a;
	}

	return -1;
}

Lump* Wad::lastLump()
{
	if (directory.size() == 0)
		return NULL;
	else
		return directory.back();
}

void Wad::swapLumps(long index1, long index2)
{
	if (!lumpAt(index1) || !lumpAt(index2))
		return;

	Lump* temp = lumpAt(index2);
	directory[index2] = lumpAt(index1);
	directory[index1] = temp;

	changed = true;
}

void Wad::swapLumps(Lump* lump1, Lump* lump2)
{
	swapLumps(lumpIndex(lump1), lumpIndex(lump2));
}

void Wad::renameDir(Lump* dirlump, string newname)
{
	for (int a = 0; a < numLumps(); a++)
	{
		Lump* lump = directory[a];

		if (lump == dirlump || lump->dirLevel() <= dirlump->dirLevel())
			continue;

		bool in = true;
		for (int d = 0; d < dirlump->dirLevel(); d++)
		{
			if (lump->getDir(d) != dirlump->getDir(d))
			{
				in = false;
				break;
			}
		}

		string dname = dirlump->getName(false);
		dname.RemoveLast();
		if (in && lump->getDir(dirlump->dirLevel()) == dname)
			lump->renameDir(newname, dirlump->dirLevel());
	}

	dirlump->setName(newname + _T("/"));
}

void Wad::deleteAllLumps()
{
	while (numLumps() > 0)
		deleteLump(0);
}
