
#include "lump_area.h"
#include <wx/listctrl.h>

class WadPanel : public wxPanel
{
private:
	Wad		wadfile;
	bool	opened_ok;

	vector<DWORD>		selection;
	wxListCtrl			*lump_list;
	LumpArea			*lump_area;
	wxStaticBoxSizer	*lump_area_sizer;
	wxImageList			*image_list;
	wxMenu				*menu_cols;

	wxColour	palette[256];
	string		palette_path;
	string		cur_dir;

public:
	enum
	{
		WP_LIST_LUMPS,

		WP_CM_RENAME,
		WP_CM_DELETE,
		WP_CM_CUT,
		WP_CM_COPY,
		WP_CM_PASTE,

		WP_COL_SIZE,
		WP_COL_TYPE,
	};

	WadPanel(wxWindow *parent, string wadpath, bool zip = false);
	~WadPanel();

	bool	isOk() { return opened_ok; }
	string	getFilename();
	Wad*	getWad() { return &wadfile; }

	void	populateLumpList();
	void	loadPalette();
	bool	closeWad();
	void	saveWad();
	void	saveWadAs();

	void	newLump();
	void	newFolder();
	void	newLumpFromFile();
	void	deleteLump();
	void	renameLump();
	void	importLump();
	void	exportLump();
	void	exportImageLump(Lump* lump);
	void	exportLumpToWad();
	void	moveLump(bool up);
	void	copyLump();
	void	pasteLump();
	void	cutLump();

	void	addLumpToList(int index, Lump* lump);
	void	updateList(int index);
	void	updateList(Lump* lump);

	void	updateSelection();
	void	updateColumnWidths();

	void	gotoNonFolder(int &index);

	void	openLump(Lump* lump);

	// Events
	void onLumpListChange(wxListEvent &event);
	void onLumpListEditName(wxListEvent &event);
	void onLumpListSelect(wxListEvent &event);
	void onLumpListDeSelect(wxListEvent &event);
	void onLumpListColResize(wxListEvent &event);
	void onLumpListActivate(wxListEvent &event);
	void onLumpListRightClick(wxListEvent &event);
	void onColumnRightClick(wxListEvent &event);

	void onContextMenuSelect(wxCommandEvent &event);

	DECLARE_EVENT_TABLE()
};

/*
class LumpTreeData : public wxTreeItemData
{
private:
	Lump*	lump;

public:
	LumpTreeData(Lump* lump) { this->lump = lump; }
	~LumpTreeData() {}

	Lump*	getLump() { return lump; }
	void	setLump(Lump* nlump) { lump = nlump; }
};
*/
